//
//  InverseKinematicsSolver.cpp
//  SimpleProg
//
//  Created by kumada on 2013/08/30.
//  Copyright (c) 2013年 kumada. All rights reserved.
//

#include "InverseKinematicsSolver.h"
#include "invert.h"
#include <cassert>
#include <boost/math/constants/constants.hpp>
#include <boost/numeric/ublas/io.hpp>

namespace ublas = boost::numeric::ublas;

const ublas::identity_matrix<double> InverseKinematicsSolver::identity_(4, 4);
const ublas::zero_matrix<double> InverseKinematicsSolver::zero_(4, 4);


InverseKinematicsSolver::InverseKinematicsSolver()
        : one_(4)



{
        axises_[X].resize(3);
        axises_[Y].resize(3);
        axises_[Z].resize(3);
        
        axises_[X][0] = 1, axises_[X][1] = 0, axises_[X][2] = 0;
        axises_[Y][0] = 0, axises_[Y][1] = 1, axises_[Y][2] = 0;
        axises_[Z][0] = 0, axises_[Z][1] = 0, axises_[Z][2] = 1;
        one_(0) = 0, one_(1) = 0, one_(2) = 0, one_(3) = 1;
}


const ublas::vector<double> InverseKinematicsSolver::six_prod(
        const ublas::matrix<double>& m0,
        const ublas::matrix<double>& m1,
        const ublas::matrix<double>& m2,
        const ublas::matrix<double>& m3,
        const ublas::matrix<double>& m4,
        const ublas::matrix<double>& m5
) const {
        const ublas::matrix<double> a = ublas::prod(m0, m1);
        const ublas::matrix<double> b = ublas::prod(a, m2);
        const ublas::matrix<double> c = ublas::prod(b, m3);
        const ublas::matrix<double> d = ublas::prod(c, m4);
        const ublas::matrix<double> e = ublas::prod(d, m5);
        
        return ublas::column(e, 3);
}

namespace
{
        constexpr double Pi2 = 2 * boost::math::constants::pi<double>();
        const double reduce(double radian)
        {
                if ( radian > Pi2 ) {
                        radian -= Pi2;
                }
                if ( radian < 0 ) {
                        radian += Pi2;
                }
                return radian;
        }
}

void InverseKinematicsSolver::execute(const ublas::vector<double>& dp, Arms& arms) const
{
        const auto a1 = six_prod(
                diff_prod(arms.s_),
                prod(arms.l_),
                prod(arms.u_),
                prod(arms.r_),
                prod(arms.b_),
                prod(arms.t_)
        );

        const auto a2 = six_prod(
                prod(arms.s_),
                diff_prod(arms.l_),
                prod(arms.u_),
                prod(arms.r_),
                prod(arms.b_),
                prod(arms.t_)
        );
        
        const auto a3 = six_prod(
                prod(arms.s_),
                prod(arms.l_),
                diff_prod(arms.u_),
                prod(arms.r_),
                prod(arms.b_),
                prod(arms.t_)
        );

        const auto a4 = six_prod(
                prod(arms.s_),
                prod(arms.l_),
                prod(arms.u_),
                diff_prod(arms.r_),
                prod(arms.b_),
                prod(arms.t_)
        );

        const auto a5 = six_prod(
                prod(arms.s_),
                prod(arms.l_),
                prod(arms.u_),
                prod(arms.r_),
                diff_prod(arms.b_),
                prod(arms.t_)
        );
        
        const auto a6 = six_prod(
                prod(arms.s_),
                prod(arms.l_),
                prod(arms.u_),
                prod(arms.r_),
                prod(arms.b_),
                diff_prod(arms.t_)
        );
        
        ublas::matrix<double> j(3, 6);
        
        j(0, 0) = a1(0), j(0, 1) = a2(0), j(0, 2) = a3(0), j(0, 3) = a4(0), j(0, 4) = a5(0), j(0, 5) = a6(0);
        j(1, 0) = a1(1), j(1, 1) = a2(1), j(1, 2) = a3(1), j(1, 3) = a4(1), j(1, 4) = a5(1), j(1, 5) = a6(1);
        j(2, 0) = a1(2), j(2, 1) = a2(2), j(2, 2) = a3(2), j(2, 3) = a4(2), j(2, 4) = a5(2), j(2, 5) = a6(2);
        assert(0 == a1(3) && 0 == a2(3) && 0 == a3(3) && 0 == a4(3) && 0 == a5(3) && 0 == a6(3));
        
        ublas::matrix<double> jt = ublas::trans(j);
        ublas::matrix<double> tmp = ublas::prod(j, jt);
        ublas::matrix<double> tmp_inv;
        math::invert(tmp, tmp_inv);
        
        ublas::matrix<double> jc = ublas::prod(jt, tmp_inv);
        ublas::vector<double> dt = ublas::prod(jc, dp);
        arms.s_.angle_ += dt(0);
        arms.l_.angle_ += dt(1);
        arms.u_.angle_ += dt(2);
        arms.r_.angle_ += dt(3);
        arms.b_.angle_ += dt(4);
        arms.t_.angle_ += dt(5);
        reduce(arms.s_.angle_);
        reduce(arms.l_.angle_);
        reduce(arms.u_.angle_);
        reduce(arms.r_.angle_);
        reduce(arms.b_.angle_);
        reduce(arms.t_.angle_);
}

