//
//  RobotData.h
//  SimpleProg
//
//  Created by kumada on 2013/08/21.
//  Copyright (c) 2013年 kumada. All rights reserved.
//

#ifndef __SimpleProg__RobotData__
#define __SimpleProg__RobotData__

#include <memory>
#include <string>
#include <list>
#include <vector>
#include "Body.h"
#include "Face.h"
#include "Vertex.h"
#include "Edge.h"
#ifdef __CYGWIN__
#include <boost/regex.hpp>
#define REGEX_NAMESPACE boost
#endif
#ifdef __APPLE__
#include <regex>
#define REGEX_NAMESPACE std
#endif
class RobotData
{
public:
        typedef std::unique_ptr<Body>   BodyPtr;
        void load(const std::string& filename);
        const Body* get_body(int i) const
        {
                return bodies_[i].get();
        }

private:
        void read_faces(Body* body, std::ifstream& ifs, std::string& line);
        void read_edges(Body* body, std::ifstream& ifs, std::string& line);
        void read_vertices(Body* body, std::ifstream& ifs, std::string& line);
        
        const std::vector<int> extract_integers(const std::string& line) const;
        const std::vector<double> extract_reals(const std::string& line) const;
        
        
        std::vector<BodyPtr>    bodies_   {};
        
        REGEX_NAMESPACE::regex  int_regex_ {R"~([-]?\d+)~"};
        REGEX_NAMESPACE::regex  real_regex_ {R"~([-+]?[0-9]+[.]?[0-9]*)~"};
};

#endif /* defined(__SimpleProg__RobotData__) */
