//
//  SimulatorProperties.h
//  SimpleProg
//
//  Created by kumada on 2013/08/24.
//  Copyright (c) 2013年 kumada. All rights reserved.
//

#ifndef SimpleProg_SimulatorProperties_h
#define SimpleProg_SimulatorProperties_h
#include "InverseKinematicsSolver.h"
#include <boost/math/constants/constants.hpp>

constexpr double Pi = boost::math::constants::pi<double>();

struct SimulatorProperties
{
        double max_angle_ = 60.0 / 180 * Pi;
        double min_angle_ = -max_angle_;
        double step_ = 10.0 / 180 * Pi;
        
        SimulatorProperties(InverseKinematicsSolver::Arms& arms)
                : arms_(arms) {}
        
        InverseKinematicsSolver::Arms& arms_;
        
        void subtract_angle(double& angle)
        {
                angle = (angle - step_);
                if ( angle < 0 ) {
                        angle += 2 * Pi;
                }
        }
        
        void add_angle(double& angle)
        {
                angle = (angle + step_);
                if ( angle > 2 * Pi ) {
                        angle -= 2 * Pi;
                }
        }
        
        void force_restriction(double& angle)
        {
                if ( angle >= max_angle_ ) {
                        angle = max_angle_;
                }
                if ( angle <= min_angle_ ) {
                        angle = min_angle_;
                }
        }
        
        void subtract_angle_with_restriction(double& angle)
        {
                angle = (angle - step_);
                force_restriction(angle);
        }
        
        void add_angle_with_restriction(double& angle)
        {
                angle = (angle + step_);
                force_restriction(angle);
        }
        
        void subtract_s()
        {
                subtract_angle(arms_.s_.angle_);
        }
        
        void add_s()
        {
                add_angle(arms_.s_.angle_);
        }

        void subtract_l()
        {
                subtract_angle_with_restriction(arms_.l_.angle_);
        }
        
        void add_l()
        {
                add_angle_with_restriction(arms_.l_.angle_);
        }

        void subtract_u()
        {
                subtract_angle_with_restriction(arms_.u_.angle_);
        }
        
        void add_u()
        {
                add_angle_with_restriction(arms_.u_.angle_);
        }

        void subtract_r()
        {
                subtract_angle(arms_.r_.angle_);
        }
        
        void add_r()
        {
                add_angle(arms_.r_.angle_);
        }

        void subtract_b()
        {
                subtract_angle_with_restriction(arms_.b_.angle_);
        }
        
        void add_b()
        {
                add_angle_with_restriction(arms_.b_.angle_);
        }

        void subtract_t()
        {
                subtract_angle(arms_.t_.angle_);
        }
        
        void add_t()
        {
                add_angle(arms_.t_.angle_);
        }
        
};
#endif
