//
//  Body.h
//  SimpleProg
//
//  Created by kumada on 2013/08/20.
//  Copyright (c) 2013年 kumada. All rights reserved.
//

#ifndef SimpleProg_Body_h
#define SimpleProg_Body_h

#include <memory>
#include <vector>
#include "Face.h"
#include "Vertex.h"
#include "Edge.h"

class Body
{
public:
        typedef std::unique_ptr<Face>   FacePtr;
        typedef std::unique_ptr<Vertex> VertexPtr;
        typedef std::unique_ptr<Edge>   EdgePtr;

        Body() = default;
        ~Body() = default;
        
        std::vector<FacePtr>& get_faces()
        {
                return faces_;
        }
        
        std::vector<EdgePtr>& get_edges()
        {
                return edges_;
        }

        std::vector<VertexPtr>& get_vertices()
        {
                return vertices_;
        }
        
        const std::vector<const Vertex*> extract_vertices(int face_index) const;
        std::size_t get_face_size() const
        {
                return faces_.size();
        }

private:
        std::vector<FacePtr>    faces_    {};
        std::vector<VertexPtr>  vertices_ {};
        std::vector<EdgePtr>    edges_    {};

        Body(const Body&) = delete;
        Body(Body&&) = delete;
        Body& operator=(const Body&) = delete;
        Body& operator=(Body&&) = delete;
        
        const Face* get_face(int i) const
        {
                return faces_[i].get();
        }
        
        const Edge* get_edge(int i) const
        {
                return edges_[i].get();
        }
        
        const Vertex* get_vertex(int i) const
        {
                return vertices_[i].get();
        }
};

#endif
